"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _KnapsackTwigRenderer_src;
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackTwigRenderer = void 0;
const app_1 = require("@knapsack/app");
const twig_renderer_1 = __importDefault(require("@basalt/twig-renderer"));
const path_1 = require("path");
const fs_extra_1 = require("fs-extra");
const utils_1 = require("./utils");
const pkg = JSON.parse((0, fs_extra_1.readFileSync)((0, path_1.join)(__dirname, '../package.json'), 'utf-8'));
app_1.log.setupUpdateNotifier(pkg);
class KnapsackTwigRenderer extends app_1.KnapsackRendererBase {
    constructor(config) {
        super({
            id: 'twig',
            language: 'twig',
            extension: '.twig',
        });
        _KnapsackTwigRenderer_src.set(this, void 0);
        this.getMeta = () => ({
            id: this.id,
            title: 'Twig',
            aliasUse: 'optional',
            aliasTitle: 'Namespaced Path',
            aliasDescription: "If using Twig Namespaces (i.e. `@components`) then provide the value you'd provide to a Twig `include`, such as `@components/card.twig`.",
            enableDataDemos: true,
            enableTemplateDemos: true,
            hasSlotsSupport: true,
            version: pkg.version,
            hasInferSpecSupport: false,
            hasTemplateSuggestionsSupport: true,
        });
        // copying b/c `TwigRenderer` mutates config!
        __classPrivateFieldSet(this, _KnapsackTwigRenderer_src, JSON.parse(JSON.stringify(config.src)), "f");
        this.twigRenderer = new twig_renderer_1.default({
            keepAlive: process.env.NODE_ENV === 'production',
            ...config,
        });
    }
    async render(opt) {
        const usage = await this.getUsage(opt);
        const results = await this.twigRenderer.renderString(usage);
        return { ...results, usage, templateLanguage: this.language };
        // return this.twigRenderer.render(template.alias, data);
    }
    async getUsage(opt) {
        const { pattern, template, demo, 
        /**
         * The Patterns class
         */
        patternManifest, } = opt;
        if (!demo) {
            return `{% include "${template.alias}" only %}`;
        }
        switch (demo.type) {
            case 'template': {
                const templateDemoPath = patternManifest.getTemplateDemoAbsolutePath({
                    patternId: pattern.id,
                    templateId: template.id,
                    demoId: demo.id,
                });
                return (0, fs_extra_1.readFile)(templateDemoPath, 'utf8');
            }
            case 'data': {
                const { data: { props, slots }, } = demo;
                if (!slots) {
                    return (0, utils_1.getTwigUsage)({
                        templateName: template.alias,
                        props,
                    });
                }
                {
                    const slotProps = {};
                    await Promise.all(Object.keys(slots).map(async (slotName) => {
                        const slottedTemplates = slots[slotName];
                        slotProps[slotName] = await Promise.all(slottedTemplates.map(async (slottedTemplate) => {
                            if (app_1.KnapsackRendererBase.isSlottedText(slottedTemplate)) {
                                return `  ${slottedTemplate.text}`;
                            }
                            if (app_1.KnapsackRendererBase.isSlottedTemplateReference(slottedTemplate)) {
                                throw new Error(`Cannot handle slotted template references in ${JSON.stringify({
                                    patternId: pattern.id,
                                    templateId: template.id,
                                    demoId: demo.id,
                                    slottedTemplate,
                                })}`);
                            }
                            const thisPattern = patternManifest.getPattern(slottedTemplate.patternId);
                            const thisTemplate = thisPattern.templates.find((t) => t.id === slottedTemplate.templateId);
                            const usage = await this.getUsage({
                                pattern: thisPattern,
                                template: thisTemplate,
                                patternManifest,
                                demo: slottedTemplate.demo ||
                                    this.patterns.demosById[slottedTemplate.demoId],
                            });
                            // indenting 2 spaces since it will be in a `{% set slotName %}` block
                            return usage
                                .split('\n')
                                .map((line) => `  ${line}`)
                                .join('\n');
                        }));
                    }));
                    const extraProps = [];
                    const slotIncludes = Object.keys(slotProps).map((slotName) => {
                        extraProps.push({
                            key: slotName,
                            value: slotName,
                        });
                        const slotCodes = slotProps[slotName];
                        return `
{% set ${slotName} %}
${slotCodes.join('\n\n')}
{% endset %}
      `.trim();
                    });
                    const usage = await (0, utils_1.getTwigUsage)({
                        templateName: template.alias,
                        props,
                        before: slotIncludes.join('\n\n'),
                        extraProps,
                    });
                    return usage;
                }
            }
            default:
                throw new Error(`Must pass in a demo.type`);
        }
    }
    async getTemplateSuggestions() {
        var _a, _b;
        const { unusedPaths } = await this.getUnusedTemplatePaths({
            globPaths: __classPrivateFieldGet(this, _KnapsackTwigRenderer_src, "f").roots.map((root) => (0, path_1.join)(root, '**/*.twig')),
            cwd: this.patterns.userConfigDir,
        });
        const namespaces = (_b = (_a = __classPrivateFieldGet(this, _KnapsackTwigRenderer_src, "f").namespaces) === null || _a === void 0 ? void 0 : _a.map(({ id, paths }) => {
            const absPaths = paths.map((path) => (0, path_1.isAbsolute)(path) ? path : (0, path_1.join)(this.patterns.userConfigDir, path));
            return {
                id,
                paths: absPaths,
            };
        })) !== null && _b !== void 0 ? _b : [];
        const suggestions = unusedPaths.map((twigPath) => {
            const namespace = namespaces.find(({ paths }) => paths.some((path) => twigPath.startsWith(path)));
            let alias;
            if (namespace) {
                namespace.paths.forEach((path) => {
                    if (twigPath.startsWith(path)) {
                        const subPath = (0, path_1.relative)(path, twigPath);
                        alias = `@${namespace.id}/${subPath}`;
                    }
                });
            }
            return {
                path: twigPath,
                alias,
            };
        });
        return { suggestions };
    }
}
exports.KnapsackTwigRenderer = KnapsackTwigRenderer;
_KnapsackTwigRenderer_src = new WeakMap();
//# sourceMappingURL=renderer-twig.js.map